! Simple program for computing the coefficient f_d of the j:j-k resonance (see Murray & Dermott 2000)
! course of Dinamica del Sistema Solare (UniPi)
program lapcoDSS
implicit none

  double precision:: alpha
  double precision:: fd
  integer:: j,k,i,ii 
  logical:: internal,ecctyp

  write(*,*) 'give positive j'
  read(*,*) j
  write(*,*) 'give positive k'
  read(*,*) k 
  write(*,*) 'eccentricity-type (1) or inclination-type (2)?'
  read(*,*) i
  write(*,*) 'internal (1) or external (2)?'
  read(*,*) ii

  ! check on order
  if(k.gt.4)then
     write(*,*) 'WARNING: maximum available order of the resonance is 4'
     STOP
  endif

  ! check on type
  if(i.eq.1)then
     ecctyp=.true.
  elseif(i.eq.2)then
     ecctyp=.false.
  else
     write(*,*) 'WARNING: wrong eccentricity/inclination type option (only entries 1 and 2 are allowed)'
     STOP
  endif

  ! check on position
  if(ii.eq.1)then
     internal=.true.
  elseif(ii.eq.2)then
     internal=.false.
  else
     write(*,*) 'WARNING: wrong internal/external option (only entries 1 and 2 are allowed)'
     STOP
  endif

  ! check on inclination type and order
  if(.not.ecctyp.and.mod(k,2).eq.1)then
     write(*,*) 'WARNING: there are not pure internal or external resonances in inclinations if order is odd'
     STOP
  endif

  if(internal.and.ecctyp)then
     write(*,*) 'internal eccentricity-type resonance', j,':',j-k 
  elseif(internal.and..not.ecctyp)then
     write(*,*) 'internal inclination-type resonance', j,':',j-k 
  elseif(.not.internal.and.ecctyp)then
     write(*,*) 'external eccentricity-type resonance', j,':',j-k 
  elseif(.not.internal.and..not.ecctyp)then
     write(*,*) 'external inclination-type resonance', j,':',j-k 
  endif    

  alpha=(REAL(j-k)/REAL(j))**(2.d0/3.d0)  ! alpha at nominal resonance

  fd=0.d0
  if(k.eq.1)then
     if(internal.and.ecctyp)then
        fd=0.5d0*(-2.d0*j*coeff_lap(0.5d0,j,0,alpha)- &
               & alpha*coeff_lap(0.5d0,j,1,alpha))
     elseif(.not.internal.and.ecctyp)then
        fd=0.5d0*((-1.d0+2.d0*j)*coeff_lap(0.5d0,j-1,0,alpha)+ &
               & alpha*coeff_lap(0.5d0,j-1,1,alpha))
        ! indirect terms
        if(j.eq.2)then
           fd=fd-0.5d0/alpha**2
        endif
     endif
  elseif(k.eq.2)then
     if(internal.and.ecctyp)then
        fd=0.125d0*((-5.d0*j+4.d0*j**2)*coeff_lap(0.5d0,j,0,alpha)+ &
               & (-2.d0+4.d0*j)*alpha*coeff_lap(0.5d0,j,1,alpha)+ & 
               & alpha**2*coeff_lap(0.5d0,j,2,alpha))
     elseif(.not.internal.and.ecctyp)then
        fd=0.125d0*((2.d0-7.d0*j+4.d0*j**2)*coeff_lap(0.5d0,j-2,0,alpha)+ &
               & (-2.d0+4.d0*j)*alpha*coeff_lap(0.5d0,j-2,1,alpha)+ & 
               & alpha**2*coeff_lap(0.5d0,j-2,2,alpha))
        ! indirect terms
        if(j.eq.3)then
           fd=fd-0.375d0/alpha**2
        endif
     elseif(internal.and..not.ecctyp)then
        fd=0.5d0*alpha*coeff_lap(1.5d0,j-1,0,alpha)
     elseif(.not.internal.and..not.ecctyp)then
        fd=0.5d0*alpha*coeff_lap(1.5d0,j-1,0,alpha)
     endif
  elseif(k.eq.3)then
     if(internal.and.ecctyp)then
        fd=1.d0/48.d0*((-26.d0*j+30.d0*j**2-8.d0*j**3)*coeff_lap(0.5d0,j,0,alpha)+ &
               & (-9.d0+27.d0*j-12.d0*j**2)*alpha*coeff_lap(0.5d0,j,1,alpha)+ &
               & (6.d0-6.d0*j)*alpha**2*coeff_lap(0.5d0,j,2,alpha)- &
               & alpha**3*coeff_lap(0.5d0,j,3,alpha))
     elseif(.not.internal.and.ecctyp)then
        fd=1.d0/48.d0*((-6.d0+29.d0*j-30.d0*j**2+8.d0*j**3)*coeff_lap(0.5d0,j-3,0,alpha)+ &
               & (6.d0-21.d0*j+12.d0*j**2)*alpha*coeff_lap(0.5d0,j-3,1,alpha)+ &
               & (-3.d0+6.d0*j)*alpha**2*coeff_lap(0.5d0,j-3,2,alpha)+ &
               & alpha**3*coeff_lap(0.5d0,j-3,3,alpha))
        ! indirect terms
        if(j.eq.4)then
           fd=fd-(1.d0/3.d0)/alpha**2
        endif      
     endif
  elseif(k.eq.4)then
     if(internal.and.ecctyp)then
        fd=1.d0/384.d0*((-206.d0*j+283.d0*j**2-120.d0*j**3+16.d0*j**4)*coeff_lap(0.5d0,j,0,alpha)+ &
               & (-64.d0+236.d0*j-168.d0*j**2+32.d0*j**3)*alpha*coeff_lap(0.5d0,j,1,alpha)+ &
               & (48.d0-78.d0*j+24.d0*j**2)*alpha**2*coeff_lap(0.5d0,j,2,alpha)+ &
               & (-12.d0+8.d0*j)*alpha**3*coeff_lap(0.5d0,j,3,alpha)+ &
               & alpha**4*coeff_lap(0.5d0,j,4,alpha))
     elseif(.not.internal.and.ecctyp)then
        fd=1.d0/384.d0*((24.d0-146.d0*j+211.d0*j**2-104.d0*j**3+16.d0*j**4)*coeff_lap(0.5d0,j-4,0,alpha)+ &
               & (-24.d0+116.d0*j-120.d0*j**2+32.d0*j**3)*alpha*coeff_lap(0.5d0,j-4,1,alpha)+ &
               & (12.d0-42.d0*j+24.d0*j**2)*alpha**2*coeff_lap(0.5d0,j-4,2,alpha)+ &
               & (-4.d0+8.d0*j)*alpha**3*coeff_lap(0.5d0,j-4,3,alpha)+ &
               & alpha**4*coeff_lap(0.5d0,j-4,4,alpha))
        ! indirect terms
        if(j.eq.5)then
           fd=fd-(125.d0/384.d0)/alpha**2
        endif  
     elseif(internal.and..not.ecctyp)then
        fd=0.375d0*(alpha**2*coeff_lap(2.5d0,j-2,0,alpha))
     elseif(.not.internal.and..not.ecctyp)then
        fd=0.375d0*(alpha**2*coeff_lap(2.5d0,j-2,0,alpha))
     endif
  endif

  print*, 'fd=',fd

contains

  double precision recursive function coeff_lap(s,j,n_der,alpha) result(val)
  implicit none

  integer, intent(in):: j, n_der
  double precision, intent(in) :: s, alpha

  double precision, parameter :: tol=1.d-10
  double precision :: r, r_tot
  integer :: k, k_max, kk
  integer :: jj

  k_max=1000

  if(j.lt.0)then                             ! simmetry between b^(j)_s e b^(-j)_s
     jj=-j
  else
     jj=j
  endif

  if(n_der.eq.0)then                         ! If no derivation, end of recursion
     val=1.d0
     if(jj.gt.0)then
        do k=0,jj-1
           val=val*(s+k)/(k+1.d0)
        enddo
     endif
     val=val*alpha**jj
     r_tot=1.d0
     do k=2,k_max
        if(mod(k,2).eq.0)then                ! only even terms
           r=1.d0
           do kk=0,k/2-1
              r=r*(s+kk)*(s+jj+kk)/((kk+1.d0)*(jj+kk+1.d0))
           enddo
           r=r*alpha**k
           r_tot=r_tot+r                     ! cumulate the result
           if(abs(r).LT.tol*r_tot)then       ! check convergence
              exit
           endif
        endif
     enddo
     val=val*r_tot
     val=val*2.d0    ! before val was 0.5*b
  elseif(n_der.eq.1)then                     ! recursive formula
     val=s*(coeff_lap(s+1,jj-1,0,alpha)-2.d0*alpha*coeff_lap(s+1,jj,0,alpha)+&
         & coeff_lap(s+1,jj+1,0,alpha))
  else                                       ! recursive formula
     val=s*(coeff_lap(s+1,jj-1,n_der-1,alpha)-2.d0*alpha*coeff_lap(s+1,jj,n_der-1,alpha)+&
         & coeff_lap(s+1,jj+1,n_der-1,alpha)-2.d0*(n_der-1.d0)*coeff_lap(s+1,jj,n_der-2,alpha))
  endif

  end function coeff_lap


end program lapcoDSS
